/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.workflow.api.jbpm.internal.cmd;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.ebpm.workflow.api.SpagicWorkflowException;
import org.eclipse.ebpm.workflow.api.Variable;
import org.hibernate.Session;
import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Property;
import org.spagic.workflow.api.jbpm.ISpagicJBPMCommand;
import org.spagic.workflow.api.jbpm.ProcessEngine;
import org.spagic.workflow.api.jbpm.ProcessInstanceExecutionContext;

public class CreateProcessInstanceCommand implements ISpagicJBPMCommand {
	
	private static Logger log = LoggerFactory.getLogger(CreateProcessInstanceCommand.class);
	private String processName = null;
	private Variable[] vars = null;
	
	public CreateProcessInstanceCommand(String processName, Variable[] vars){
		this.processName = processName;
		this.vars = vars;
	}
	
	public Object execute(ProcessEngine processEngine, JbpmContext jbpmContext,
			Session aSession, ProcessInstanceExecutionContext instanceExecutionContext) {
	
			org.spagic.metadb.model.Process spagicMetaDBProcess = ProcessEngine.getLastDefinitionForProcess(aSession,
					processName);
			
			Property standaloneProperty = AuditDBUtils.getProcessProperty(aSession, spagicMetaDBProcess, "IsStandalone");
			if (standaloneProperty != null && standaloneProperty.getValue().equalsIgnoreCase("false")) {
				throw new SpagicWorkflowException(" Process [" + processName + "] Cannot Be Istantiated As Standalone Process");
			}
			
			GraphSession graphSession = jbpmContext.getGraphSession();
			ProcessDefinition processDefinition = graphSession.findLatestProcessDefinition(processName);

			// With the processDefinition that we retrieved from the database,
			// we can create an execution of the process definition just like in
			// the hello world example (which was without persistence).
			org.jbpm.graph.exe.ProcessInstance jbpmProcessInstance = null;
			if (processDefinition != null)
				if (this.vars != null){
					Map jbpmVars = new HashMap();
						for (Variable v : vars) {
							Object toStoreValue = v.getValue();
							if (ProcessEngine.isSerializationNeeded(v.getValue())){
								toStoreValue = ProcessEngine.serializeValue(toStoreValue);
								jbpmVars.put(ProcessEngine.createFlagVariable(v.getName()),"TRUE");
							}
							
							jbpmVars.put(v.getName(), toStoreValue);
						}
					jbpmProcessInstance = processDefinition.createProcessInstance(jbpmVars);
				} else{
					jbpmProcessInstance = processDefinition.createProcessInstance();
				}

			// Create the Spagic process instance in the Spagic metadb
			org.spagic.metadb.model.ProcessInstance spagicProcessInstance = ProcessEngine.createSpagicProcess(
					jbpmProcessInstance, null, aSession);
			
			if (spagicProcessInstance == null) {
				log.error("Problems registering Spagic on the database");

				throw new SpagicWorkflowException("Cannot Create Spagic Process on Database");
			} else {
				// Create process instance global variables
				ProcessEngine.createSpagicVariables(spagicProcessInstance, jbpmProcessInstance);

				// Now the processInstance is saved in the database. So the
				// current state of the execution of the process is stored in the
				// database.
				jbpmContext.save(jbpmProcessInstance);

				long spagicProcessInstanceID = spagicProcessInstance.getIdProcessInstance();
				
				return spagicProcessInstanceID;
			}
		}
}
