/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic.workflow.api.ws.client;

import it.eng.spagosoa.utils.SOAPUtils;

import java.util.List;

import javax.activation.DataHandler;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPMessage;

import org.apache.commons.lang.StringEscapeUtils;
import org.eclipse.ebpm.workflow.api.IControlAPI;
import org.eclipse.ebpm.workflow.api.IControlExtendedAPI;
import org.eclipse.ebpm.workflow.api.IProfileService;
import org.eclipse.ebpm.workflow.api.IQueryAPI;
import org.eclipse.ebpm.workflow.api.SpagicWorkflowException;
import org.spagic.workflow.command.CMDControlAPI;
import org.spagic.workflow.command.CMDControlExtendedAPI;
import org.spagic.workflow.command.CMDQueryAPI;
import org.spagic.workflow.command.CmdObject;
import org.spagic.workflow.command.ICommandBasedProcessEngine;
import org.spagic.workflow.command.ICommandSerializer;
import org.spagic.workflow.command.ResultObject;
import org.spagic.workflow.command.XStreamCommandSerializer;

public class WSProcessEngine implements ICommandBasedProcessEngine {

	private IControlAPI controlAPI = null;
	private IControlExtendedAPI controlExtendedAPI = null;
	private IQueryAPI queryAPI = null;
	private String wfEngineURL = null;

	public WSProcessEngine(String wfEngineURL) {
		super();
		this.wfEngineURL = wfEngineURL;
		controlAPI = new CMDControlAPI(this);
		controlExtendedAPI = new CMDControlExtendedAPI(this);
		queryAPI = new CMDQueryAPI(this);
	}

	public IControlAPI getControlAPI() {

		return this.controlAPI;
	}

	public IControlExtendedAPI getControlExtendedAPI() {

		return this.controlExtendedAPI;
	}

	public IQueryAPI getQueryAPI() {

		return this.queryAPI;
	}

	public void stop() {

	}

	public Object executeCmdObject(CmdObject cmdObject) {
		try {
			ICommandSerializer commandSerializer = new XStreamCommandSerializer(); 
			String streamedXMLCommand = commandSerializer.asXML(cmdObject);
			SOAPMessage soapRequest = SOAPUtils.createSOAPMessageFromString("<payload xmlns=\"urn:eng:spagic\">" + StringEscapeUtils.escapeXml(streamedXMLCommand) + "</payload>");
			SOAPMessage soapResponse = SOAPUtils.call(wfEngineURL, soapRequest);
			if (soapResponse.getSOAPBody().getFault() != null) {
				SOAPFault fault = soapResponse.getSOAPBody().getFault();
				throw new SpagicWorkflowException(fault.getFaultCode() + " " + fault.getFaultString());
			}
			org.w3c.dom.Node nodeChild = soapResponse.getSOAPBody().getFirstChild().getFirstChild();
			String streamedXMLResponse = null;
			if (nodeChild instanceof org.apache.axis.message.Text) {
				streamedXMLResponse = nodeChild.toString();
			} else {
				streamedXMLResponse = nodeChild.getTextContent();
			}
			//System.out.println("Streamed XML Response " + streamedXMLResponse);
			ResultObject resultObject = commandSerializer.fromXML(streamedXMLResponse);
			List<DataHandler> attachments = SOAPUtils.getAttachments(soapResponse);
			resultObject.setAttachments(attachments);
			if (resultObject.getErrorsOccured()) {
				throw new SpagicWorkflowException(resultObject.getErrorMessage());
			} else {
				return resultObject;
			}
		} catch (SpagicWorkflowException e) {
			throw e;
		} catch (Throwable t) {
			t.printStackTrace();
			throw new RuntimeException(t.getMessage(), t);
		}
	}

	public String getWfEngineURL() {
		return wfEngineURL;
	}

	public void setWfEngineURL(String wfEngineURL) {
		this.wfEngineURL = wfEngineURL;
	}

	
	public void setProfileService(IProfileService profileService) {
		//do noting this is only a client stub that delegates
		//request to remote service		
	}
	
	public IProfileService getProfileService(){
		return null;
	}
		//do noting this is only a client stub that del

}
