/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.workflow.command;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.fdf.FDFDocument;
import org.eclipse.ebpm.workflow.api.IQueryAPI;
import org.eclipse.ebpm.workflow.api.Process;
import org.eclipse.ebpm.workflow.api.ProcessInstance;
import org.eclipse.ebpm.workflow.api.SubProcessTask;
import org.eclipse.ebpm.workflow.api.Task;
import org.eclipse.ebpm.workflow.api.TaskInstance;
import org.eclipse.ebpm.workflow.api.Variable;
import org.w3c.dom.Document;

public class CMDQueryAPI implements IQueryAPI {
	public static final String PROCESS_INSTANCE_ID = "processInstanceId";
	private ICommandBasedProcessEngine engine = null;

	public CMDQueryAPI(ICommandBasedProcessEngine engine) {
		this.engine =  engine;
	}

	public Task[] getHumanTask(long processId) {
		String method = CmdObject.GET_HUMAN_TASKS_BY_PROCESS_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Task[]) resultObject.getResult();
	}

	@Override
	public TaskInstance[] getHistoricListofTasks(long processInstanceId) {
		String method = CmdObject.GET_HISTORIC_LIST_OF_TASKS;

		Map<String, Object> paramMap = new HashMap<String, Object>();

		CmdObject cmdObject = new CmdObject(method, paramMap);
		paramMap.put(PROCESS_INSTANCE_ID, processInstanceId);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}


	public Task[] getHumanTask(String processName) {
		String method = CmdObject.GET_HUMAN_TASKS_BY_PROCESS_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Task[]) resultObject.getResult();
	}

	public Task[] getHumanTask(Process process) {

		return getHumanTask(process.getProcessId());
	}

	public TaskInstance[] getHumanTaskInstances(long processInstanceId) {
		String method = CmdObject.GET_HUMAN_TASKS_INSTANCES_FOR_PROCESS_INSTANCE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	public TaskInstance[] getHumanTaskInstances(long processInstanceId, String attributeName, String attributeValue) {
		String method = CmdObject.GET_HUMAN_TASKS_INSTANCES_FOR_PROCESS_INSTANCE_FILTERED;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("attributeName", attributeName);
		paramMap.put("attributeValue", attributeValue);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	public ProcessInstance[] getLinkedProcessInstances(long processInstanceId) {
		String method = CmdObject.GET_LINKED_PROCESS_INSTANCES_FOR_PROCESS_INSTANCE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance[]) resultObject.getResult();
	}

	public Process getProcess(long processId) {
		String method = CmdObject.GET_PROCESS_BY_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process) resultObject.getResult();
	}

	public Process getProcess(String processName) {
		String method = CmdObject.GET_PROCESS_BY_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process) resultObject.getResult();
	}

	public String getProcessGraph(long processId) {
		String method = CmdObject.GET_PROCESS_GRAPH;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String) resultObject.getResult();
	}

	public ProcessInstance getProcessInstanceById(long processInstanceId) {
		String method = CmdObject.GET_PROCESS_INSTANCE_BY_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance) resultObject.getResult();
	}

	public String getProcessInstanceGraph(long processInstanceId) {
		String method = CmdObject.GET_PROCESS_INSTANCE_GRAPH;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String) resultObject.getResult();
	}

	public String getProcessInstanceStatus(long processInstanceId) {
		String method = CmdObject.GET_PROCESS_INSTANCE_STATUS;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String) resultObject.getResult();
	}

	public ProcessInstance[] getProcessInstances() {
		String method = CmdObject.GET_ALL_PROCESSES_INSTANCES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance[]) resultObject.getResult();
	}

	public ProcessInstance[] getProcessInstances(long processId) {
		String method = CmdObject.GET_ALL_PROCESSES_INSTANCES_FOR_PROCESS_BY_ID;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance[]) resultObject.getResult();
	}

	public ProcessInstance[] getProcessInstances(String processName) {
		String method = CmdObject.GET_ALL_PROCESSES_INSTANCES_FOR_PROCESS_BY_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance[]) resultObject.getResult();
	}

	public Process[] getProcesses() {
		String method = CmdObject.GET_ALL_PROCESSES;

		Map<String, Object> paramMap = new HashMap<String, Object>();

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public Process[] getProcessesByFilter(String attributeName, String attributeValue, boolean onlyStandalone) {
		String method = CmdObject.GET_ALL_PROCESSES_FILTERED_BY;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("attributeName", attributeName);
		paramMap.put("attributeValue", attributeValue);
		paramMap.put("onlyStandalone", onlyStandalone);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public ProcessInstance[] getSubProcessInstances(long parentProcessInstanceId) {
		String method = CmdObject.GET_SUBPROCESS_INSTANCES_BY_PARENT_PROCESS_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("parentProcessInstanceId", parentProcessInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (ProcessInstance[]) resultObject.getResult();
	}

	public Process[] getSubProcesses(long processId) {
		String method = CmdObject.GET_SUBPROCESS_BY_PROCESS_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public Process[] getSubProcesses(String processName) {
		String method = CmdObject.GET_SUBPROCESS_BY_PROCESS_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public String getTaskInstanceStatus(long processInstanceId, long taskInstanceId) {

		String method = CmdObject.GET_HUMAN_TASK_INSTANCE_STATUS;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskInstanceId", taskInstanceId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String) resultObject.getResult();
	}

	public String[] getHumanTaskOutgoingValues(String processName, String taskName) {
		String method = CmdObject.GET_HUMAN_TASK_OUTGOING_VALUES_BY_PROCESS_NAME_AND_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		paramMap.put("taskName", taskName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String[]) resultObject.getResult();
	}

	public String[] getHumanTaskOutgoingValues(long processId, String taskName) {
		String method = CmdObject.GET_HUMAN_TASK_OUTGOING_VALUES_BY_PROCESS_ID_AND_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);
		paramMap.put("taskName", taskName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String[]) resultObject.getResult();
	}

	public Process[] getLatestDefinitionProcesses() {
		String method = CmdObject.GET_LATEST_DEFINITION_PROCESSES;

		Map<String, Object> paramMap = new HashMap<String, Object>();

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public Process[] getLatestDefinitionProcesses(boolean onlyStandalone) {
		String method = CmdObject.GET_LATEST_DEFINITION_PROCESSES_ONLY_STANDALONE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("onlyStandalone", onlyStandalone);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public Process[] getLatestDefinitionProcessesByFilter(String attributeName, String attributeValue,
			boolean onlyStandalone) {

		String method = CmdObject.GET_LATEST_DEFINITION_PROCESSES_BY_FILTER;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("attributeName", attributeName);
		paramMap.put("attributeValue", attributeValue);
		paramMap.put("onlyStandalone", onlyStandalone);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public Process[] getProcesses(boolean onlyStandalone) {
		String method = CmdObject.GET_ALL_PROCESSES_ONLY_STANDALONE;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("onlyStandalone", onlyStandalone);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Process[]) resultObject.getResult();
	}

	public SubProcessTask[] getSubprocessTasks(long processId) {
		String method = CmdObject.GET_SUBPROCESS_TASKS_BY_PROCESS_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (SubProcessTask[]) resultObject.getResult();
	}

	public SubProcessTask[] getSubprocessTasks(String processName) {
		String method = CmdObject.GET_SUBPROCESS_TASKS_BY_PROCESS_NAME;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (SubProcessTask[]) resultObject.getResult();
	}

	public SubProcessTask[] getSubprocessTasks(Process process) {
		return getSubprocessTasks(process.getProcessId());
	}

	public String[] getNextTasksNames(long processInstanceId) {
		String method = CmdObject.GET_NEXT_TASKS_NAMES;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (String[]) resultObject.getResult();
	}

	
	public DataHandler getTaskDocumentation(String processName, String taskName) {
		String method = CmdObject.GET_TASK_DOCUMENTATION;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		paramMap.put("taskName", taskName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		List<DataHandler> attachments = resultObject.getAttachments();
		DataHandler result = null;
		if (attachments != null && attachments.size() > 0)
			result = attachments.get(0);
		return result;
	}

	

	public ICommandBasedProcessEngine getEngine() {
		return engine;
	}

	

	public TaskInstance[] getGeneratedHumanTaskInstances(String processName, String taskName, Date createFrom,
			Date createTo, Date dueDateFrom, Date dueDateTo, int state, long processInstanceId) {
		String method = CmdObject.GET_GENERATED_HUMAN_TASKS_INSTANCES_FILTERED;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		paramMap.put("taskName", taskName);
		paramMap.put("createFrom", createFrom);
		paramMap.put("createTo", createTo);
		paramMap.put("dueDateFrom", dueDateFrom);
		paramMap.put("dueDateTo", dueDateTo);
		paramMap.put("state", state);
		paramMap.put("processInstanceId", processInstanceId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	public Task getHumanTask(long processId, String taskName) {
		String method = CmdObject.GET_HUMAN_TASK_BY_PROCESS_ID_AND_TASK_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);
		paramMap.put("taskName", taskName);

		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Task) resultObject.getResult();
	}

	
	public TaskInstance[] getGeneratedHumanTaskInstances() {
		
			String method = CmdObject.GET_GENERATED_HUMAN_TASKS_INSTANCES;

			Map<String, Object> paramMap = new HashMap<String, Object>();

			CmdObject cmdObject = new CmdObject(method, paramMap);
			ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
			return (TaskInstance[]) resultObject.getResult();
		}

	
	public TaskInstance[] getHumanTaskInstancesForActor(String actorId) {
		String method = CmdObject.GET_HUMAN_TASKS_INSTANCES_FOR_ACTOR;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}


	public TaskInstance[] getHumanTaskInstancesForActorAndProcess(long processInstanceId, String actorId){
			String method = CmdObject.GET_HUMAN_TASKS_INSTANCES_FOR_PROCESS_INSTANCE_AND_ACTOR;

			Map<String, Object> paramMap = new HashMap<String, Object>();
			paramMap.put("processInstanceId", processInstanceId);
			paramMap.put("actorId", actorId);

			CmdObject cmdObject = new CmdObject(method, paramMap);
			ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
			return (TaskInstance[]) resultObject.getResult();
	}

	@Override
	public Variable[] getProcessProperties(long processId) {
		String method = CmdObject.GET_PROCESS_PROPERTIES_BY_ID;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Variable[]) resultObject.getResult();
	}

	@Override
	public Variable[] getProcessProperties(String processName) {
		String method = CmdObject.GET_PROCESS_PROPERTIES_BY_NAME;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processName", processName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Variable[]) resultObject.getResult();
	}

	@Override
	public TaskInstance[] getHumanTaskInstancesAssignedToActor(String actorId) {
		String method = CmdObject.GET_HUMAN_TASK_INSTANCES_ASSIGNED_TO_ACTOR;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	@Override
	public TaskInstance[] getGeneratedHumanTaskInstances(long processInstanceId) {
		String method = CmdObject.GET_GENERATED_HUMAN_TASKS_INSTANCES;

		Map<String, Object> paramMap = new HashMap<String, Object>();

		CmdObject cmdObject = new CmdObject(method, paramMap);
		paramMap.put(PROCESS_INSTANCE_ID, processInstanceId);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	@Override
	public TaskInstance[] getHumanTaskInstancesForActor(String actorId, Variable[] variables) {
		String method = CmdObject.GET_HUMAN_TASK_INSTANCES_ASSIGNED_TO_ACTOR_WITH_VARS;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("actorId", actorId);
		paramMap.put("variables", variables);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (TaskInstance[]) resultObject.getResult();
	}

	@Override
	public PDDocument getEmptyPDF(String pdfURL) {
		String method = CmdObject.GET_EMPTY_PDF;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("pdfURL", pdfURL);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (PDDocument) resultObject.getResult();
	}

	@Override
	public PDDocument getFilledPDF(String pdfURL, FDFDocument fdf, String sendButtonName) {
		String method = CmdObject.GET_FILLED_PDF;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("pdfURL", pdfURL);
		paramMap.put("fdf", fdf);
		paramMap.put("sendButtonName", sendButtonName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (PDDocument) resultObject.getResult();
	}

	@Override
	public FDFDocument getFDFfromDB(long processInstanceId, String taskName) {
		String method = CmdObject.GET_FDF_FROM_DB;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processInstanceId", processInstanceId);
		paramMap.put("taskName", taskName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (FDFDocument) resultObject.getResult();
	}

	@Override
	public DataHandler getDataHandlerFromPDF(PDDocument pdf, String fileName) {
		String method = CmdObject.GET_DATAHANDLER_FROM_PDF;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("pdf", pdf);
		paramMap.put("fileName", fileName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (DataHandler) resultObject.getResult();
	}

	@Override
	public DataHandler fillDocument(String sendButtonName, Map<String, Properties> fields, String urlToSend, String pdfURL, Document inMsgDoc) {
		String method = CmdObject.FILL_PDF_DOCUMENT;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("sendButtonName", sendButtonName);
		paramMap.put("fields", fields);
		paramMap.put("urlToSend", urlToSend);
		paramMap.put("pdfURL", pdfURL);
		paramMap.put("inMsgDoc", inMsgDoc);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		List<DataHandler> attachments = resultObject.getAttachments();
		DataHandler dataHandler = null;
		for (Iterator iterator = attachments.iterator(); iterator.hasNext();) {
			dataHandler = (DataHandler) iterator.next();		
		}
		return dataHandler;
	}

	@Override
	public Map<String, String> getTaskAttributes(long processId, String taskName) {
		String method = CmdObject.GET_TASK_ATTRIBUTES;

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("processId", processId);
		paramMap.put("taskName", taskName);
		CmdObject cmdObject = new CmdObject(method, paramMap);
		ResultObject resultObject = (ResultObject) this.engine.executeCmdObject(cmdObject);
		return (Map<String, String>) resultObject.getResult();
	}

	
	
}
