/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.workflow.jbpm.authservice;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.eclipse.ebpm.workflow.api.IProfileService;

public class FileBasedAuthorizationService implements IProfileService {
	
	private Map<String, List<String>> userRolesMap = null;
	
	public FileBasedAuthorizationService(File f){
		try{
			SAXReader reader = new SAXReader();
			Document doc = reader.read(f);
			
			List<Node> users = doc.selectNodes("/AUTHORIZATIONS/ENTITIES/USERS/*");
			List<Node> behaviours = doc.selectNodes("/AUTHORIZATIONS/RELATIONS/BEHAVIOURS/*");
			userRolesMap = new HashMap<String, List<String>>();
			for (Node user: users) {
				String name = user.valueOf("@userID");
				List<String> roles = new ArrayList<String>();
				for (Node behaviour: behaviours) {
					String userid = behaviour.valueOf("@userID");
					String role = behaviour.valueOf("@roleName");
					if (userid.equals(name)) {
						roles.add(role);
					}
				}
			if (roles.size() > 0)
				userRolesMap.put(name, roles);
		}
	} catch (Exception e) {
		e.printStackTrace();
		throw new RuntimeException(e);
	}
	}
	
	public List<String> getUserRoles(String actorId) { 
		 return userRolesMap.get(actorId);
	}

	public List<String> getUsers() {
		List<String> users = new ArrayList<String>();
		users.addAll(userRolesMap.keySet());
		return users;
	}

	
	/**
	 * Init the.
	*/
	@SuppressWarnings("unchecked")
	private void init() {
		
		
	}

}
